`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/02 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

`define ENABLE 1'b1
module topmodule(
    D_ssd,
    ssd_ctl,
    clk,
    rst_n,
    select_AM_PM,
    select_clk,
    select_clk_DIP,
    select_sec_min,
    select_month_year,
    out_LED_speed
    );

    output [7:0] D_ssd;
    output [3:0] ssd_ctl;
    output [3:0] out_LED_speed;
    input clk;
    input rst_n;
    input select_AM_PM;
    input select_clk;
    input select_sec_min;
    input select_month_year; // use DIP switch
    input select_clk_DIP; //use DIP switch
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire clk_10000Hz;
    wire [1:0] clk_ssd_ctl;

    wire debounced_in_AM_PM, debounced_in_clk, debounced_in_sec_min;
    wire out_pulse_AM_PM, out_pulse_clk, out_pulse_sec_min;

    wire curr_state_clk, curr_state_AM_PM, curr_state_sec_min;
    wire clk_out_for_count;
    
    wire AM_PM_convert; 
    wire enable_minute;
    wire enable_day;

    wire [3:0] digit_3_hour, digit_2_hour, digit_1_min, digit_0_min;

    wire [3:0] digit_1_mon, digit_0_mon;
    wire [3:0] digit_1_day, digit_0_day;
    wire [3:0] digit_3_year, digit_2_year, digit_1_year, digit_0_year;
    wire [3:0] digit_1_sec, digit_0_sec;
    wire [3:0] hour_digit_0, hour_digit_1;

    wire [3:0] ssd_in_month, ssd_in_year, ssd_in_sec, ssd_in_min;
    wire [3:0] ssd_ctl_month, ssd_ctl_year, ssd_ctl_sec, ssd_ctl_min;

    wire [7:0] D_ssd_a, D_ssd_b;

    wire [3:0] ssd_in;
    //wire [3:0] ssd_ctl_a, ssd_ctl_b;
    //wire [3:0] ssd_in_a, ssd_in_b;
   

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_10000Hz(clk_10000Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst_n(rst_n)   //low-active reset
    );

    debounce U1( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_AM_PM),
    .pb_debounced(debounced_in_AM_PM)
    );

    debounce U11( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_clk),
    .pb_debounced(debounced_in_clk)
    );

    debounce U111( 
    .rst(rst_n), 
    .clk(clk_100Hz), 
    .pb_in(select_sec_min),
    .pb_debounced(debounced_in_sec_min)
    );

    one_pulse U2( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in_AM_PM),
    .out_pulse(out_pulse_AM_PM)
);
    one_pulse U22( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in_clk),
    .out_pulse(out_pulse_clk)
);
    one_pulse U222( 
    .rst_n(rst_n), 
    .clk(clk_100Hz), 
    .in_trig(debounced_in_sec_min),
    .out_pulse(out_pulse_sec_min)
);
    
    fsm U3(
    .select_mode_change(out_pulse_AM_PM),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state_AM_PM)
);
    fsm U33(
    .select_mode_change(out_pulse_clk),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state_clk)
    );

    fsm U333(
    .select_mode_change(out_pulse_sec_min),
    .clk(clk_100Hz),
    .rst_n(rst_n),
    .current_state(curr_state_sec_min)
    );

    select_clk U3333(
    .clk_1Hz(clk_1Hz),
    .clk_100Hz(clk_100Hz),
    .clk_10000Hz(clk_10000Hz),
    .clk_crystal(clk),
    .clk_out_for_count(clk_out_for_count),
    .select(curr_state_clk),
    .select_use_DIP(select_clk_DIP),
    .out_LED_speed(out_LED_speed)
);
    BCD_up_2d U33333(
    .digit_1(digit_1_sec),
    .digit_0(digit_0_sec),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .carry_out(enable_minute)
    );

    BCD_up_4d U333333(
    .digit_3(digit_3_hour),
    .digit_2(digit_2_hour),
    .digit_1(digit_1_min),
    .digit_0(digit_0_min),
    .rst_n(rst_n),
    .carry_out(enable_day),
    .clk(clk_out_for_count),
    .AM_PM_convert(AM_PM_convert),
    .en(enable_minute)
    );

    leap_year U05(
    .leap_year(leap_year),
    .rst_n(rst_n),
    .clk(digit_0_year[0])
    );

    BCD_up_month_date U4(
    .digit_3(digit_1_mon),
    .digit_2(digit_0_mon),
    .digit_1(digit_1_day),
    .digit_0(digit_0_day),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .en(enable_day), 
    .cout(enable_year),
    .leap_year(leap_year)
    );

    BCD_up_4d_year U5(
    .digit_3(digit_3_year),
    .digit_2(digit_2_year),
    .digit_1(digit_1_year),
    .digit_0(digit_0_year),
    .rst_n(rst_n),
    .clk(clk_out_for_count),
    .en(enable_year)
    );

    

    AM_PM_selector U55(
    .hour_in_1(digit_3_hour),
    .hour_in_0(digit_2_hour),
    .hour_out_1(hour_digit_1),
    .hour_out_0(hour_digit_0),
    .AM_PM_convert(AM_PM_convert),
    .select_mode(curr_state_AM_PM)
    );

    scan_freq_ctl U555( 
    .ssd_in(ssd_in_sec),
    .ssd_ctl(ssd_ctl_sec),
    .in_0(digit_0_sec),
    .in_1(digit_1_sec),
    .in_2(4'd0),
    .in_3(4'd0),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );

    scan_freq_ctl U5555(
    .ssd_in(ssd_in_min),
    .ssd_ctl(ssd_ctl_min),
    .in_0(digit_0_min),
    .in_1(digit_1_min),
    .in_2(hour_digit_0),
    .in_3(hour_digit_1),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );
   
    scan_freq_ctl U6(
    .ssd_in(ssd_in_month),
    .ssd_ctl(ssd_ctl_month),
    .in_0(digit_0_day),
    .in_1(digit_1_day),
    .in_2(digit_0_mon),
    .in_3(digit_1_mon),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );

    scan_freq_ctl U61(
    .ssd_in(ssd_in_year),
    .ssd_ctl(ssd_ctl_year),
    .in_0(digit_0_year),
    .in_1(digit_1_year),
    .in_2(digit_2_year),
    .in_3(digit_3_year),
    .clk_ctl_en(clk_ssd_ctl),
    .rst_n(rst_n)
    );
  
    ssd_selector U1111(
    .ssd_ctl(ssd_ctl),
    .ssd_in(ssd_in),
    .ssd_ctl_month(ssd_ctl_month),
    .ssd_ctl_year(ssd_ctl_year),
    .ssd_ctl_min(ssd_ctl_min),
    .ssd_ctl_sec(ssd_ctl_sec),
    .ssd_in_month(ssd_in_month),
    .ssd_in_year(ssd_in_year),
    .ssd_in_min(ssd_in_min),
    .ssd_in_sec(ssd_in_sec),
    .select_month_year(select_month_year),
    .select_sec_min(curr_state_sec_min)
);

    display U8(.D_ssd(D_ssd_a), .i(ssd_in));

    display_with_point U9(.D_ssd(D_ssd_b), .i(ssd_in));

    display_selector U10(
    .D_ssd(D_ssd),
    .D_ssd_a(D_ssd_a),
    .D_ssd_b(D_ssd_b),
    .select_mode(curr_state_AM_PM),
    .AM_PM_convert(AM_PM_convert),
    .clk_ssd_ctl(clk_ssd_ctl)
    );

    

    
endmodule
